#!/bin/bash

#*******************************************************************************
# script to flash u-boot image.
# The script will upgrade the u-boot only if the image version is grater than
# currently running u-boot version.
#*******************************************************************************

# The function writes the u-boot image to the flash.
write_uboot_image() 
{
   ntries=0
   
   while [ "$ntries" -lt 3 ]; do
      if [ ! -e "$uboot_image" ]; then
         echo "`date +%c` [u-boot_instl] Missing uboot image file $uboot_image"
         exit 1
      fi

      /nn/bin/eraseall /dev/mtd7
      /bin/cat $uboot_image | /nn/bin/eengine -1 > /dev/mtd7
      /bin/cat /dev/mtd7 | /nn/bin/eengine -1 > /tmp/uboot.image
      /usr/bin/od --strings=6 /tmp/uboot.image | /bin/grep "$uboot_image_version"
      nreturn=$?
      if [ $nreturn -eq 0 ]; then
         echo "`date +%c` [u-boot_instl] u-boot flash image updated"
         break
      else
         /nn/bin/eraseall /dev/mtd7
         /bin/cat $uboot_image | /nn/bin/eengine -1 > /dev/mtd7
         (( ntries += 1 ))
      fi
    done

    if [ "$ntries" -eq 3 ]; then
      echo "`date +%c` [u-boot_instl] Error: uboot flash image NOT updated successfully"
      exit 1
    fi
}


# The fn gets the proper u-boot image file based on the CSC board type.
get_board_type()
{
   if [ -f /proc/nn/boardtype ] && [ "`cat /proc/nn/boardtype`" == "2" ]; then
      uboot_image=/nn/images/uboot/csc_r2_u-boot.bin
   else
      uboot_image=/nn/images/uboot/csc_r1_u-boot.bin
   fi
}


# The function gets the version of the currently running u-boot and the u-boot image file
get_version()
{
   if [ -f "$uboot_image" ]
   then
      uboot_image_version=`/usr/bin/od --strings=6 $uboot_image | grep "CSC U-Boot" | awk -F"CSC" '{print $2}' | awk '{print $1,$2}'`
   else
      echo "`date +%c` [u-boot_instl] Image [$uboot_image] not found. exiting !"
      exit 1
   fi

   if [ -f /nn/bin/fw_printenv ]
   then
      /bin/cat /dev/mtd7 | /nn/bin/eengine -1 > /tmp/runtime_uboot.img
 uboot_runtime_version=`/usr/bin/od --strings=6 /tmp/runtime_uboot.img | grep "CSC U-Boot" | awk -F"CSC" '{print $2}' | awk '{print $1,$2}'`
   else
      echo "`date +%c` [u-boot_instl] fw_printenv cmd not found. exiting !"
      exit 1
   fi
}

#**************************************************************************
# Start of Main
#**************************************************************************

# First get the u-boot image file name based on the CSC board version.
uboot_image=""
get_board_type

if [ "$uboot_image" = "" ]
then
   echo "`date +%c` [u-boot_instl] Cannot determaine u-boot image name, exiting !"
   exit 1
fi

# get the u-boot versions, if it's not possible to get any of the version then exit.
uboot_image_version=""
uboot_runtime_version=""
get_version

if [[ -z "$uboot_image_version" ]]
then
   echo "`date +%c` [u-boot_instl] Cannot determine u-boot image version, [image=$uboot_image_version, runtime=$uboot_runtime_version]. exiting !"
   exit 1
fi

# Compare the u-boot version in the image and in the flash.
# Proceed only if the image version is greater
if [[ "$uboot_image_version" > "$uboot_runtime_version" ]]
then
   echo "`date +%c` [u-boot_instl] Upgrading u-boot[$uboot_image], [image=$uboot_image_version, runtime=$uboot_runtime_version]."
   write_uboot_image
else
   echo "`date +%c` [u-boot_instl] u-boot version is up-to-date, not installing the u-boot. exit !"
   exit 1
fi   

/bin/rm -f /tmp/runtime_uboot.img 
/bin/rm -f /tmp/uboot.image 

# If we reach here, then it means u-boot was flashed successfully.
# Just return 0, don't exit(). Because exit() will exit from the main patch installation script also
#return 0

